<?php

/**
 * MyBB 1.4
 * Copyright © 2008 MyBB Group, All Rights Reserved
 *
 * ProfileComments by Santiago Dimattia
 * http://www.teleportz.com.ar
 * 
 */

if(!defined("IN_MYBB"))
{
	die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
}

$plugins->add_hook("admin_formcontainer_end", "profilecomments_edit_group");
$plugins->add_hook("admin_user_groups_edit_commit", "profilecomments_edit_group_do");
$plugins->add_hook("member_profile_end", "profilecomments_page");
$plugins->add_hook("global_start", "profilecomments_make_comments_as_read", 5);
$plugins->add_hook("global_start", "profilecomments_new_comments");

function profilecomments_info()
{
	/**
	 * Array of information about the plugin.
	 * name: The name of the plugin
	 * description: Description of what the plugin does
	 * website: The website the plugin is maintained at (Optional)
	 * author: The name of the author of the plugin
	 * authorsite: The URL to the website of the author (Optional)
	 * version: The version number of the plugin
	 * guid: Unique ID issued by the MyBB Mods site for version checking
	 * compatibility: A CSV list of MyBB versions supported. Ex, "121,123", "12*". Wildcards supported.
	 */
	return array(
				"name" => "هاك تعليقات الملف الشخصي",
				"description" => "يعطي الصلاحية للأعضاء أن يضعوا تعليقات في الملف الشخصي. <p>تعريب <a href=\"http://www.al-wrsha.co.cc/\">منتديات الورشة التطويرية</a>",
				"website" => "http://mods.mybboard.net/view/profile-comments/",
				"author" => "Santiago Dimattia",
				"authorsite" => "http://www.teleportz.com.ar",
				"version" => "0.6.0",
				"guid" => "e6063fe5a0fd9b7b7982f68336bf1d4d",
				"compatibility" => "14*,16*"
			);
}

function profilecomments_install()
{
	global $lang, $db, $cache;
	
	// Load the language
	$lang->load('profilecomments');

	// Creates the table where comments are saved
	$tables = "CREATE TABLE `".TABLE_PREFIX."profilecomments` (
  `mid` int(10) NOT NULL auto_increment,
  `user` int(10) NOT NULL,
  `sender` int(10) NOT NULL,
  `text` text NOT NULL,
  `date` int(10) NOT NULL,
  PRIMARY KEY  (`mid`)
) ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=18 ;";
	$db->write_query($tables);

	// Creates fields for groups-based permissions
	$db->write_query("ALTER TABLE `".TABLE_PREFIX."usergroups` ADD `canmanagecomments` INT(1) NOT NULL DEFAULT '0';");
	$db->write_query("ALTER TABLE `".TABLE_PREFIX."usergroups` ADD `cansendcomments` INT(1) NOT NULL DEFAULT '0';");
	$db->write_query("ALTER TABLE `".TABLE_PREFIX."usergroups` ADD `caneditselfcomments` INT(1) NOT NULL DEFAULT '0';");
	$db->write_query("ALTER TABLE `".TABLE_PREFIX."usergroups` ADD `candeleteselfcomments` INT(1) NOT NULL DEFAULT '0';");
	
	// Give access to send and edit own comments to ALL USERGROUPS
	$db->write_query('UPDATE '.TABLE_PREFIX.'usergroups SET canmanagecomments = "0", cansendcomments = "1", caneditselfcomments = "1"');
	// Give access to EDIT & DELETE (all comments) to Administrators
	$db->write_query('UPDATE '.TABLE_PREFIX.'usergroups SET canmanagecomments = "1", candeleteselfcomments = "1" WHERE gid = "4"');
	// Delete all access for guests
	$db->write_query('UPDATE '.TABLE_PREFIX.'usergroups SET cansendcomments = "0", caneditselfcomments = "0" WHERE gid = "1"');
	// Delete all access for banned users
	$db->write_query('UPDATE '.TABLE_PREFIX.'usergroups SET cansendcomments = "0", caneditselfcomments = "0" WHERE gid = "7"');
	// Delete all access for 'awaiting activation' users
	$db->write_query('UPDATE '.TABLE_PREFIX.'usergroups SET cansendcomments = "0", caneditselfcomments = "0" WHERE gid = "5"');
	
	// Update usergroups cache
	$cache->update_usergroups();

	// Creates a field in users table (For new comments alert)
	$db->write_query("ALTER TABLE `".TABLE_PREFIX."users` ADD `newcomments` INT(10) NOT NULL DEFAULT '0';");

	// Create the config group for MyBB Settings
	$query = $db->simple_select("settinggroups", "COUNT(*) as rows");
	$rows = $db->fetch_field($query, "rows");

	$new_groupconfig = array(
		'name' => 'profilecomments', 
		'title' => $db->escape_string($lang->profile_comments),
		'description' => $db->escape_string($lang->profile_comments_info),
		'disporder' => $rows+1,
		'isdefault' => 0
	);
	
	$group['gid'] = $db->insert_query("settinggroups", $new_groupconfig);

	// Insert ProfileComments options
	$new_config = array();
	
	$new_config[] = array(
		'name' => 'profilecomments_perpage',
		'title' => $db->escape_string($lang->profile_comments_perpage),
		'description' => $db->escape_string($lang->profile_comments_perpage_info),
		'optionscode' => 'text',
		'value' => '4',
		'disporder' => 1,
		'gid' => $group['gid']
	);
	
	$new_config[] = array(
		'name' => 'profilecomments_alertbar',
		'title' => $db->escape_string($lang->profile_comments_alertbar),
		'description' => $db->escape_string($lang->profile_comments_alertbar_info),
		'optionscode' => 'yesno',
		'value' => '0',
		'disporder' => 2,
		'gid' => $group['gid']
	);
	
	$new_config[] = array(
		'name' => 'profilecomments_alertbar_text',
		'title' => $db->escape_string($lang->profile_comments_alertbar_text),
		'description' => $db->escape_string($lang->profile_comments_alertbar_text_info),
		'optionscode' => 'text',
		'value' => $db->escape_string($lang->profile_comments_alertbar_text_content),
		'disporder' => 3,
		'gid' => $group['gid']
	);
	
	$new_config[] = array(
		'name' => 'profilecomments_alertbar_style',
		'title' => $db->escape_string($lang->profile_comments_alertbar_style),
		'description' => $db->escape_string($lang->profile_comments_alertbar_style_info),
		'optionscode' => 'text',
		'value' => '',
		'disporder' => 4,
		'gid' => $group['gid']
	);
	
	$new_config[] = array(
		'name' => 'profilecomments_default_avatar',
		'title' => $db->escape_string($lang->profile_comments_default_avatar),
		'description' => $db->escape_string($lang->profile_comments_default_avatar_info),
		'optionscode' => 'text',
		'value' => $db->escape_string('images/avatars/invalid_url.gif'),
		'disporder' => 5,
		'gid' => $group['gid']
	);

	$new_config[] = array(
		'name' => 'profilecomments_allow_html',
		'title' => $db->escape_string($lang->profile_comments_allow_html),
		'description' => $db->escape_string($lang->profile_comments_allow_html_info),
		'optionscode' => 'yesno',
		'value' => '0',
		'disporder' => 6,
		'gid' => $group['gid']
	);

	$new_config[] = array(
		'name' => 'profilecomments_allow_smilies',
		'title' => $db->escape_string($lang->profile_comments_allow_smilies),
		'description' => $db->escape_string($lang->profile_comments_allow_smilies_info),
		'optionscode' => 'yesno',
		'value' => '1',
		'disporder' => 7,
		'gid' => $group['gid']
	);

	$new_config[] = array(
		'name' => 'profilecomments_allow_bbcode',
		'title' => $db->escape_string($lang->profile_comments_allow_bbcode),
		'description' => $db->escape_string($lang->profile_comments_allow_bbcode_info),
		'optionscode' => 'yesno',
		'value' => '1',
		'disporder' => 8,
		'gid' => $group['gid']
	);
	
		$new_config[] = array(
		'name' => 'profilecomments_allow_imgcode',
		'title' => $db->escape_string($lang->profile_comments_allow_imgcode),
		'description' => $db->escape_string($lang->profile_comments_allow_imgcode_info),
		'optionscode' => 'yesno',
		'value' => '1',
		'disporder' => 9,
		'gid' => $group['gid']
	);
	
	$new_config[] = array(
		'name' => 'profilecomments_filter_badwords',
		'title' => $db->escape_string($lang->profile_comments_filter_badwords),
		'description' => $db->escape_string($lang->profile_comments_filter_badwords_info),
		'optionscode' => 'yesno',
		'value' => '1',
		'disporder' => 10,
		'gid' => $group['gid']
	);

	$new_config[] = array(
		'name' => 'profilecomments_show_editor',
		'title' => $db->escape_string($lang->profile_comments_show_editor),
		'description' => $db->escape_string($lang->profile_comments_show_editor_info),
		'optionscode' => 'yesno',
		'value' => '0',
		'disporder' => 10,
		'gid' => $group['gid']
	);
	
	foreach($new_config as $array => $content)
	{
		$db->insert_query("settings", $content);
	}
	
	rebuild_settings();
}

function profilecomments_uninstall()
{
	global $db, $cache;

	// Delete table where comments are saved
	$db->write_query('DROP TABLE `'.TABLE_PREFIX.'profilecomments`');

	// Delete fields for groups-based permissions
	$db->write_query("ALTER TABLE `".TABLE_PREFIX."usergroups` DROP `canmanagecomments`;");
	$db->write_query("ALTER TABLE `".TABLE_PREFIX."usergroups` DROP `cansendcomments`;");
	$db->write_query("ALTER TABLE `".TABLE_PREFIX."usergroups` DROP `caneditselfcomments`;");
	$db->write_query("ALTER TABLE `".TABLE_PREFIX."usergroups` DROP `candeleteselfcomments`;");

	// Delete field for new comments alert
	$db->write_query("ALTER TABLE `".TABLE_PREFIX."users` DROP `newcomments`;");

	// Update usergroups cache
	$cache->update_usergroups();

	// Delete config groups
	$query = $db->simple_select('settinggroups', 'gid', 'name = "profilecomments"');
	$groupid = $db->fetch_field($query, 'gid');
	$db->delete_query('settings','gid = "'.$groupid.'"');
	$db->delete_query('settinggroups','gid = "'.$groupid.'"');

	rebuild_settings();
}

function profilecomments_is_installed()
{
	global $db;

	// If table exist, plugin is installed
	if($db->table_exists('profilecomments'))
	{
		return TRUE;
	}

	return FALSE;
}

function profilecomments_activate()
{
	global $db;

	// Create new templates
	$new_template['profile_comments_alert'] = '<div class="pm_alert" id="comment_notice" style="{$mybb->settings[\'profilecomments_alertbar_style\']}">
	<div>
		<a href="{$mybb->settings[\'bburl\']}/member.php?action=profile&amp;uid={$mybb->user[\'uid\']}" title="{$lang->profile_comments}">{$commentalert_text}</a>
	</div>
</div>
<br />';

	$new_template['profile_comments'] = '{$comments_form_edit}
<br />
<table border="0" cellspacing="{$theme[\'borderwidth\']}" cellpadding="{$theme[\'tablespace\']}" class="tborder">
	<thead>
		<tr>
			<td colspan="2" class="thead">
				<div class="expcolimage">
					<img src="{$theme[\'imgdir\']}/{$hide_image}" id="comments_img" class="expander" alt="{$hide_text}" title="{$hide_text}" />
				</div>
				<div>
					<strong>{$lang->profile_comments_list}</strong>
				</div>
			</td>
		</tr>
	</thead>
	<tbody style="{$hidecomments}" id="comments_e">
		{$comments_form}
		{$comments}
	</tbody>
</table>
{$pagination}';

	$new_template['profile_comments_delete'] = '- <a href="member.php?action=profile&uid={$uid}&op=delete&todo={$id}" title="{$lang->profile_comments_delete_comment}">{$lang->profile_comments_delete}</a>';

	$new_template['profile_comments_edit'] = '- <a href="member.php?action=profile&uid={$uid}&op=edit&todo={$id}" title="{$lang->profile_comments_edit_comment}">{$lang->profile_comments_edit}</a>';

	$new_template['profile_comments_form'] = '<form id="newcomment" name="newcomment" action="member.php?action=profile&amp;uid={$uid}&amp;op=send" method="post">
<tr>
	<td colspan="2" style="text-align: left;" class="trow2">
		<div id="replyingto" class="pm_alert" style="display: none;"></div>
		<p>
			<textarea name="message" id="message" rows="10" cols="70" tabindex="2"></textarea>
		</p>
		<p>
			<input type="hidden" name="reply_to_profile" id="reply_to_profile" value="default" />
			<input type="submit" value="{$lang->profile_comments_add_button}" />
		</p>
		{$editor}
		<script type="text/javascript">
			Event.observe(window, \'load\', function() {

				var editorloaded = {$editor_active};

				$$("a.reply").each(function(a)
				{
					a.observe(\'click\', function(c) {
						var username = $(this).readAttribute(\'username\');

						if(editorloaded == 1)
						{
							clickableEditor.performInsert(\'[b]\' + username + \'[/b], \', \'\', true, false);
						}
						else
						{
							$(\'message\').value += \'[b]\' + username + \'[/b], \';
						}

						Event.stop(c);
					});
				});


				$$("a.reply_to_profile").each(function(a)
				{
					a.observe(\'click\', function(c) {
						var username = $(this).readAttribute(\'username\');
						var uid = $(this).readAttribute(\'userid\');
						$(\'reply_to_profile\').value = uid;

						if(editorloaded == 1)
						{
							//clickableEditor.performInsert(\'[b]\' + username + \'[/b], \', \'\', true, false);
						}
						else
						{
							//$(\'message\').value += \'[b]\' + username + \'[/b], \';
						}

						$("replyingto").update(\'You are replying to \' + username + \'. <a href="{$profile_page}" id="linkcancel">Cancel.</a>\').show();

						$(\'linkcancel\').observe(\'click\', function(c) {
							$("replyingto").hide();
							$(\'reply_to_profile\').value = \'default\';
							Event.stop(c);
						});

						Event.stop(c);
					});
				});
			});
		</script>
	</td>
</tr>
</form>';

	$new_template['profile_comments_form_edit'] = '<br />
<table border="0" cellspacing="{$theme[\'borderwidth\']}" cellpadding="{$theme[\'tablespace\']}" class="tborder">
	<thead>
		<tr>
			<td colspan="2" class="thead">
				<div>
					<strong>{$lang->profile_comments_editing}</strong>
				</div>
			</td>
		</tr>
	</thead>
	<tbody style="{$hidecomments}" id="comments_e">
		<form id="newcomment" name="newcomment" action="member.php?action=profile&amp;uid={$uid}&amp;op=edit&amp;todo={$todo}" method="post">
			<tr>
				<td colspan="2" class="trow2">
					<p>
						<textarea name="editmessage" id="editmessage" rows="10" cols="70" tabindex="2">{$row[\'text\']}</textarea>
					</p>
					<p>
						<input type="submit" value="{$lang->profile_comments_edit_button}" />
					</p>
				</td>
			</tr>
		</form>
	</tbody>
</table>';

	$new_template['profile_comments_list'] = '<tr id="comment-{$id}">
	<td class="{$style}" rowspan="2" width="100" style="text-align: center; vertical-align: top;">
		<img style="width: 90px;" src="{$avatar}" alt="{$username_plain} " />
	</td>
	<td class="{$style}" >
		{$username} <small style="font-size: 10px;">({$date} {$lang->at} {$time})</small><br />
		<span style="font-size: 10px;">
			{$reply} {$edit} {$delete}
		</span>
	</td>
</tr>
<tr>
	<td class="{$style}" >
		{$text}
	</td>
</tr>';

	$new_template['profile_comments_nocomments'] = '<tr>
	<td class="trow1" colspan="2">
		{$lang->profile_comments_none_found}
	</td>
</tr>';

	$new_template['profile_comments_reply'] = '<a href="#" class="reply" username="{$row[\'username\']}" title="{$lang->profile_comments_reply}">{$lang->profile_comments_reply}</a> - <a href="#" username="{$row[\'username\']}" userid="{$row[\'sender\']}" class="reply_to_profile" title="{$lang->profile_comments_reply_to_profile}">{$lang->profile_comments_reply_to_profile}</a>';

	foreach($new_template as $title => $template)
	{
		$new_template = array('title' => $db->escape_string($title), 'template' => $db->escape_string($template), 'sid' => '-1', 'version' => '140', 'dateline' => TIME_NOW);
		$db->insert_query('templates', $new_template);
	}

	// Edit some existing templates
	require MYBB_ROOT."/inc/adminfunctions_templates.php";

	find_replace_templatesets('member_profile', '#{\$modoptions}#', '{\$modoptions}<!-- ProfileComments -->{$comments_index}<!-- /ProfileComments -->');

	find_replace_templatesets('header_welcomeblock_member', '#{\$lang->welcome_pms_usage}#', '{$lang->welcome_pms_usage}<!-- ProfileComments --> | <a href="{$mybb->settings[\'bburl\']}/member.php?action=profile&uid={$mybb->user[\'uid\']}" title="{$lang->profile_comments}">{$lang->profile_comments_new_inmenu}</a> {$lang->profile_comments_new_inmenu_count}<!-- /ProfileComments -->');

	find_replace_templatesets('header', '#{\$unreadreports}#', '{$unreadreports}<!-- ProfileComments -->
			{$commentalert}<!-- /ProfileComments -->');
}


function profilecomments_deactivate()
{
	global $db;

	// Delete created templates
	$delete_templates = array('profile_alert', 'profile_comments', 'profile_comments_delete', 'profile_comments_edit', 'profile_comments_reply', 'profile_comments_form', 'profile_comments_form_edit', 'profile_comments_form_edut', 'profile_comments_list', 'profile_comments_nocomments');

	foreach($delete_templates as $template)
	{
		$db->delete_query('templates', "title = '{$template}'");
	}

	// And delete ProfileComments content from edited templates
	require_once MYBB_ROOT."/inc/adminfunctions_templates.php";
	
	find_replace_templatesets('member_profile', '#\<!--\sProfileComments\s--\>(.+)\<!--\s/ProfileComments\s--\>#is', '', 0);
	find_replace_templatesets('header_welcomeblock_member', '#\<!--\sProfileComments\s--\>(.+)\<!--\s/ProfileComments\s--\>#is', '', 0);
	find_replace_templatesets('header', '#\<!--\sProfileComments\s--\>(.+)\<!--\s/ProfileComments\s--\>#is', '', 0);
}

// Add group-based permission
function profilecomments_edit_group()
{
	global $run_module, $form_container, $lang, $form, $mybb;

	// If the current tab is "Users & Permissions", add plugin options
	if($run_module == 'user' && !empty($form_container->_title) & !empty($lang->users_permissions) & $form_container->_title == $lang->users_permissions)
	{
		// Load the language
		$lang->load('profilecomments');

		// And generate the checkbox
		$profilecomments_options = array();
		$profilecomments_options[] = $form->generate_check_box('canmanagecomments', 1, $lang->profile_comments_can_manage, array('checked' => $mybb->input['canmanagecomments']));
		$profilecomments_options[] = $form->generate_check_box('cansendcomments', 1, $lang->profile_comments_can_send, array('checked' => $mybb->input['cansendcomments']));
		$profilecomments_options[] = $form->generate_check_box('caneditselfcomments', 1, $lang->profile_comments_can_selfedit, array('checked' => $mybb->input['caneditselfcomments']));
		$profilecomments_options[] = $form->generate_check_box('candeleteselfcomments', 1, $lang->profile_comments_can_selfdelete, array('checked' => $mybb->input['candeleteselfcomments']));
		
		$form_container->output_row($lang->profile_comments, '', '<div class="group_settings_bit">'.implode('</div><div class="group_settings_bit">', $profilecomments_options).'</div>');
	}
}

// Update group-bassed permission
function profilecomments_edit_group_do()
{
	global $updated_group, $mybb;

	$updated_group['canmanagecomments'] = $mybb->input['canmanagecomments'];
	$updated_group['cansendcomments'] = $mybb->input['cansendcomments'];
	$updated_group['caneditselfcomments'] = $mybb->input['caneditselfcomments'];
	$updated_group['candeleteselfcomments'] = $mybb->input['candeleteselfcomments'];
}

function profilecomments_page()
{
	global $mybb, $cache, $db, $templates, $lang, $parser, $theme, $collapsed, $comments_form, $comments_index, $uid, $row;
	
	// Load the language
	$lang->load('profilecomments');

	// Set $uid var
	if($mybb->input['uid'])
	{
		$uid = intval($mybb->input['uid']);
	}
	else
	{
		$uid = $mybb->user['uid'];
	}
	
	// Set logged user uid
	$this_user = $mybb->user['uid'];
	
	// ProfilePage URL
	$profile_page = 'member.php?action=profile&uid='.$uid;
	
	// Set user permissions
	$permissions['admin'] = $mybb->usergroup['canmanagecomments'];
	$permissions['send'] = $mybb->usergroup['cansendcomments'];
	$permissions['edit'] = $mybb->usergroup['caneditselfcomments'];
	$permissions['delete'] = $mybb->usergroup['candeleteselfcomments'];
	
	// Set some global vars
	$message = $db->escape_string($mybb->input['message']);
	$action = $mybb->input['op'];
	$todo = intval($mybb->input['todo']);
	
	// Hide comments table
	if(isset($collapsed['comments_c']) && $collapsed['comments_c'] == "display: show;")
	{
		$hide_image = 'collapse_collapsed.gif';
		$hidecomments = 'display: none;';
		$hide_text = "[+]";
	}
	else
	{
		$hide_image = 'collapse.gif';
		$hidecomments = '';
		$hide_text = "[-]";
	}
	
	// Warning: Cool things happen below this line (?)
	
	// If user is banned, deactivate profile comments
	$usergroups = $cache->read('usergroups');
	
	$query = $db->simple_select('users', 'usergroup', 'uid = ' . $uid);
	$group = $db->fetch_field($query, 'usergroup');
	
	if($usergroups[$group]['isbannedgroup'])
	{
		// TODO: "Sorry, you can't add a comment because this user is banned"?
		return false;
	}
	
	// If adding a new comment
	if($action == 'send')
	{
		// User can't send comments
		if($permissions['send'] != 1)
		{
			error_no_permission();
		}
		
		// If the message is empty
		if(empty($message))
		{
			redirect($profile_page, $lang->profile_comments_insert_comment);
		}
		
		// If user has used "Reply to Profile" option, change the user uid
		if($mybb->input['reply_to_profile'] != 'default' && is_numeric($mybb->input['reply_to_profile']))
		{
			$uid = intval($mybb->input['reply_to_profile']);
		}
		
		// Insert the message
		$insert = array(
			'user' => $uid,
			'sender' => $this_user,
			'text' => $message,
			'date' => TIME_NOW
		);
		$query = $db->insert_query('profilecomments', $insert);
		
		// Active 'New message' alert
		if($this_user != $uid)
		{
			$update = array(
				'newcomments' => 1
			);
			$query = $db->update_query('users', $update, 'uid = ' . $uid);
		}

		// If user has sent the message using "Reply to Profile" option
		if($mybb->input['reply_to_profile'] != 'default')
		{
			redirect('member.php?action=profile&uid=' . intval($mybb->input['reply_to_profile']), $lang->profile_comments_added);
		}
		
		// Redirect to current user profile page
		redirect($profile_page, $lang->profile_comments_added);
	}
	
	if($action == 'delete')
	{
		$query = $db->simple_select('profilecomments', 'COUNT(sender) AS sender', 'sender = '.$this_user.' AND mid = '.$todo, array('limit' => 1));
		$sender = $db->fetch_field($query, "sender");
		
		// If user can delete comments
		if(($permissions['delete'] == 1 && $this_user == $sender) OR $permissions['admin'] == 1)
		{
			// Delete comment
			$db->delete_query('profilecomments', 'mid = '.$todo, 1);
		
			// Redirect to profile page
			redirect($profile_page, $lang->profile_comments_deleted);
		}
		else
		{
			error_no_permission();
		}
	}
	
	if($action == 'edit')
	{
		$editmessage = $db->escape_string($mybb->input['editmessage']);
		
		$query = $db->query('SELECT * FROM '.TABLE_PREFIX.'profilecomments LEFT JOIN '.TABLE_PREFIX.'users ON '.TABLE_PREFIX.'users.uid = '.TABLE_PREFIX.'profilecomments.sender WHERE '.TABLE_PREFIX.'profilecomments.mid = '.$todo);
		
		// If the comment exists
		if($db->num_rows($query) > 0)
		{
			$row = $db->fetch_array($query);

			if(($permissions['edit'] == 1 && $this_user == $row['sender']) OR $permissions['admin'] == 1)
			{
				// If $message isn't empty, we need to modify the comment :)
				if(!empty($editmessage))
				{
					$update = array(
						'text' => $editmessage
					);
					$query = $db->update_query('profilecomments', $update, 'mid = ' . $todo, 1);
				}
				else
				{
					$lang->profile_comments_editing = $lang->sprintf($lang->profile_comments_editing, $row['mid'], $row['username']);
					eval("\$comments_form_edit .= \"".$templates->get("profile_comments_form_edit")."\";");			
				}
			}
			else
			{
				error_no_permission();
			}
			
		}
		else
		{
			error($lang->profile_comments_invalid_id);
		}
	}
	
	// Query the number of comments
	$query = $db->simple_select('profilecomments', 'COUNT(mid) AS comments', 'user = '.$uid, array('limit' => 1));
	$quantity = $db->fetch_field($query, "comments");

	// In wich page the user is?
	$page = intval($mybb->input['page']);

	// Set how much comments show per page
	$perpage = intval($mybb->settings['profilecomments_perpage']);

	// Pagination magic
	if($page > 0)
	{
		$start = ($page - 1) * $perpage;
		$pages = $quantity / $perpage;
		$pages = ceil($pages);
		if($page > $pages || $page <= 0)
		{
			$start = 0;
			$page = 1;
		}
	}
	else
	{
		$start = 0;
		$page = 1;
	}

	// Select comments from the DB
	$query = $db->query('SELECT * FROM ' . TABLE_PREFIX . 'profilecomments LEFT JOIN ' . TABLE_PREFIX . 'users ON ' . TABLE_PREFIX . 'users.uid = ' . TABLE_PREFIX . 'profilecomments.sender LEFT JOIN ' . TABLE_PREFIX . 'usergroups ON ' . TABLE_PREFIX . 'users.usergroup = ' . TABLE_PREFIX . 'usergroups.gid WHERE ' . TABLE_PREFIX . 'profilecomments.user = ' . $uid . ' ORDER BY date DESC LIMIT ' . $start . ', ' . $perpage);

	if($db->num_rows($query) > 0)
	{
		// Show pagination
		$pagination = multipage($quantity, $perpage, $page, $profile_page);
		
		// Set table style
		$style = "trow1";
	
		while($row = $db->fetch_array($query))
		{
			// Set user-friendly ID var
			$id = $row['mid'];
			
			// Format date
			$date = my_date($mybb->settings['dateformat'], $row['date']);
			$time = my_date($mybb->settings['timeformat'], $row['date']);

			// Format and link username
			$username = format_name($row['username'], $row['usergroup'], $row['displaygroup']);
			$username = build_profile_link($username, $row['sender']);
	
			// Default avatar
			$avatar = (!empty($row['avatar'])) ? $row['avatar'] : $mybb->settings['profilecomments_default_avatar'];
			
			// Parse bbcode
			$text_parser = array(
				"allow_html" => intval($mybb->settings['profilecomments_allow_html']),
				"allow_mycode" => intval($mybb->settings['profilecomments_allow_bbcode']),
				"allow_smilies" => intval($mybb->settings['profilecomments_allow_smilies']),
				"allow_imgcode" => intval($mybb->settings['profilecomments_allow_imgcode']),
				"filter_badwords" => intval($mybb->settings['profilecomments_filter_badwords'])
			);
			$text = $parser->parse_message($row['text'], $text_parser);
			
			// If user can edit own comments or is admin
			$edit = '';
			if(($permissions['edit'] == 1 && $this_user == $row['sender']) OR $permissions['admin'] == 1)
			{
				eval("\$edit = \"".$templates->get("profile_comments_edit")."\";");
			}
			
			// If user can delete own comments or is admin
			$delete = '';
			if(($permissions['delete'] == 1 && $this_user == $row['sender']) OR $permissions['admin'] == 1)
			{
				eval("\$delete = \"".$templates->get("profile_comments_delete")."\";");
			}
			
			// If user can send comments & is not editing a comment
			$reply = '';
			if($permissions['send'])
			{
				// Add "Reply" & "Reply to Profile" links
				eval("\$reply = \"".$templates->get("profile_comments_reply")."\";");
			}
			
			eval("\$comments .= \"".$templates->get("profile_comments_list")."\";");
			
			// Change table style
			$style = alt_trow();
		}
	
	}
	else
	{
		// There is no comment to show
		eval("\$comments .= \"".$templates->get("profile_comments_nocomments")."\";");
	}

	// If user can send comments & is not editing a comment
	if($permissions['send'])
	{
		// Show the ADD NEW COMMENT form
		$lang->profile_comments_editing = $lang->sprintf($lang->profile_comments_editing, $row['mid'], $row['username']);
		
		if($mybb->settings['profilecomments_show_editor'] == 1)
		{
			$editor = build_mycode_inserter();
			$editor_active = 1;
		}
		else
		{
			$editor = '';
			$editor_active = 0;
		}
		
		eval("\$comments_form .= \"".$templates->get("profile_comments_form")."\";");
	}

	eval("\$comments_index .= \"".$templates->get("profile_comments")."\";");
}

function profilecomments_new_comments()
{
	global $mybb, $db, $templates, $lang, $commentalert, $commentalert_text, $comment_inmenu;

	// Load the language
	$lang->load('profilecomments');
	
	// Set New Comments var
	$new_comments = $mybb->user['newcomments'];

	// If the user is logged in
	if($mybb->user['uid'] != 0 && $new_comments > 0)
	{
		// Show the numer "(x)" in the header
		$lang->profile_comments_new_inmenu_count = $lang->sprintf($lang->profile_comments_new_inmenu_count, $new_comments);
		
		// If alert bar is active, show it
		if($mybb->settings['profilecomments_alertbar'] == 1)
		{
			$commentalert_text = $lang->sprintf($mybb->settings['profilecomments_alertbar_text'], $new_comments);
			eval("\$commentalert .= \"".$templates->get("profile_comments_alert")."\";");
		}
	}
	else
	{
		$lang->profile_comments_new_inmenu_count = '';
	}
}

function profilecomments_make_comments_as_read()
{
	global $mybb, $db;
	// Check if user viewing own profile
	if(THIS_SCRIPT != 'member.php')
	{
		return FALSE;
	}
	
	if(!empty($mybb->input['uid']) && $mybb->input['uid'] != $mybb->user['uid'])
	{
		return false;
	}
	
	// Update last visit
	if($mybb->user['newcomments'])
	{
		$update = array(
			'newcomments' => 0
		);
		$query = $db->update_query('users', $update, 'uid = ' . $mybb->user['uid'], 1);
		
		$mybb->user['newcomments'] = 0;
	}
}